package limax.auany;

import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;

import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;

import limax.auany.json.InviteCode;
import limax.codec.CharSource;
import limax.codec.JSONEncoder;
import limax.codec.SinkStream;
import limax.defines.ErrorCodes;
import limax.defines.ErrorSource;
import limax.endpoint.AuanyService.Result;
import limax.util.ElementHelper;

class Invite {
	private static long LIFETIME;
	private final static RuntimeException exception = new RuntimeException();
	private final static Map<Long, Long> map0 = new LinkedHashMap<>();
	private final static Map<Long, Long> map1 = new LinkedHashMap<>();

	private static void cleanup(long now) {
		for (Iterator<Map.Entry<Long, Long>> it = map0.entrySet().iterator(); it.hasNext()
				&& now - it.next().getValue() > LIFETIME;)
			it.remove();
		for (Iterator<Map.Entry<Long, Long>> it = map1.entrySet().iterator(); it.hasNext()
				&& now - it.next().getValue() > LIFETIME;)
			it.remove();
	}

	private static long alloc(int appId) {
		synchronized (exception) {
			while (true) {
				long now = System.nanoTime();
				cleanup(now);
				long code = Long.MIN_VALUE | ((long) Long.hashCode(now) << 32) | appId;
				if (map0.putIfAbsent(code, now) == null)
					return code;
			}
		}
	}

	private static boolean test0(long code) {
		synchronized (exception) {
			long now = System.nanoTime();
			cleanup(now);
			if (map0.remove(code) == null)
				return false;
			map1.put(code, now);
			return true;
		}
	}

	public static boolean test1(long code) {
		synchronized (exception) {
			cleanup(System.nanoTime());
			return map1.remove(code) != null;
		}
	}

	public static void init(Element e, Map<String, HttpHandler> httphandlers) {
		ElementHelper eh = new ElementHelper(e);
		LIFETIME = eh.getLong("lifetime", 60000l) * 1000000l;
		httphandlers.put("/invite", new HttpHandler() {
			@Override
			public void handle(HttpExchange exchange) throws IOException {
				OutputStream os = exchange.getResponseBody();
				Headers responseHeaders = exchange.getResponseHeaders();
				URI uri = exchange.getRequestURI();
				try {
					String query = URLDecoder.decode(uri.getQuery(), "UTF-8");
					int pos = query.indexOf("=");
					int appId = Integer.parseInt(query.substring(pos + 1));
					InviteCode ic = new InviteCode(alloc(Integer.parseInt(query.substring(pos + 1))),
							AppManager.randomSwitcher(
									AppManager.ServiceType.valueOf(query.substring(0, pos).toUpperCase()), appId));
					responseHeaders.set("Content-Type", "application/json; charset=utf-8");
					responseHeaders.set("Connection", "close");
					responseHeaders.set("Access-Control-Allow-Origin", "*");
					exchange.sendResponseHeaders(HttpURLConnection.HTTP_OK, 0);
					JSONEncoder.encode(ic, new CharSource(StandardCharsets.UTF_8, new SinkStream(os)));
				} catch (Exception e) {
					exchange.sendResponseHeaders(HttpURLConnection.HTTP_BAD_REQUEST, 0);
				} finally {
					os.close();
				}
			}
		});
	}

	public static void check(String username, String token, Set<Integer> pvids, Result result) {
		try {
			long code = Long.parseLong(username);
			if (pvids.size() != 1 || pvids.iterator().next() != ServiceManagerListener.getInstance().getProviderId())
				throw exception;
			if (!test0(code))
				throw exception;
			result.apply(ErrorSource.LIMAX, ErrorCodes.SUCCEED, username);
		} catch (Throwable t) {
			result.apply(ErrorSource.LIMAX, ErrorCodes.AUANY_AUTHENTICATE_FAIL, "");
		}
	}
}
