package limax.xmlgen.java;

import java.io.PrintStream;
import java.util.Collection;

import limax.util.StringUtils;
import limax.xmlgen.Bean;
import limax.xmlgen.Bind;
import limax.xmlgen.Cbean;
import limax.xmlgen.Type;
import limax.xmlgen.TypeAny;
import limax.xmlgen.TypeBinary;
import limax.xmlgen.TypeBoolean;
import limax.xmlgen.TypeByte;
import limax.xmlgen.TypeDouble;
import limax.xmlgen.TypeFloat;
import limax.xmlgen.TypeInt;
import limax.xmlgen.TypeList;
import limax.xmlgen.TypeLong;
import limax.xmlgen.TypeMap;
import limax.xmlgen.TypeSet;
import limax.xmlgen.TypeShort;
import limax.xmlgen.TypeString;
import limax.xmlgen.TypeVector;
import limax.xmlgen.Variable;
import limax.xmlgen.Visitor;
import limax.xmlgen.Xbean;

class Copy implements Visitor {
	private final PrintStream ps;
	private final String varname;
	private final String thisVar;
	private final String prefix;

	private static void make(Collection<Variable> variables, String className, String paramType, PrintStream ps,
			String prefix) {
		ps.println(prefix + "public " + className + "(" + paramType + " _o_) {");
		for (Variable var : variables)
			var.getType().accept(new Copy(var, ps, prefix + "	"));
		ps.println(prefix + "}");
		ps.println();
	}

	public static void make(Bind bind, PrintStream ps, String prefix) {
		make(bind.getVariables(), bind.getName(), TypeName.getName(bind.getTable().getValueType()), ps, prefix);
	}

	private Copy(Variable var, PrintStream ps, String prefix) {
		this.ps = ps;
		this.prefix = prefix;
		this.varname = var.getName();
		this.thisVar = "this." + this.varname;
	}

	private void model1() {
		ps.println(prefix + thisVar + " = _o_.get" + StringUtils.upper1(varname) + "();");
	}

	@Override
	public void visit(Bean type) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void visit(TypeByte type) {
		model1();
	}

	@Override
	public void visit(TypeShort type) {
		model1();
	}

	@Override
	public void visit(TypeInt type) {
		model1();
	}

	@Override
	public void visit(TypeLong type) {
		model1();
	}

	@Override
	public void visit(TypeFloat type) {
		model1();
	}

	@Override
	public void visit(TypeDouble type) {
		model1();
	}

	@Override
	public void visit(TypeString type) {
		model1();
	}

	private void copyCollection(Type type, Type valuetype) {
		String typeName = TypeName.getName(type);
		ps.println(prefix + thisVar + " = new " + typeName + "();");
		if (valuetype instanceof Cbean || valuetype instanceof Xbean) {
			ps.println(prefix + "for (" + TypeName.getBoxingName(valuetype) + " _v_ : _o_.get"
					+ StringUtils.upper1(varname) + "())");
			ps.println(prefix + "	" + thisVar + ".add(_v_);");
		} else if (valuetype.isConstType()) {
			ps.println(prefix + thisVar + ".addAll(_o_.get" + StringUtils.upper1(varname) + "());");
		} else
			throw new RuntimeException("unkown type in collection: " + valuetype.getClass().getName());
	}

	private String copyBaseMapType(Type type, String localvarname) {
		String boxingName = TypeName.getBoxingName(type);
		if (type instanceof Cbean || type instanceof Xbean)
			return "new " + boxingName + "(" + localvarname + ")";
		if (type.isConstType())
			return localvarname;
		throw new RuntimeException("unkown type in map");
	}

	private void copyBaseMap(Type type, Type keyType, Type valueType) {
		String typeName = TypeName.getName(type);
		ps.println(prefix + thisVar + " = new " + typeName + "();");
		String KV = "<" + TypeName.getBoxingName(keyType) + ", " + TypeName.getBoxingName(valueType) + ">";
		String keycopy = copyBaseMapType(keyType, "_e_.getKey()");
		String valuecopy = copyBaseMapType(valueType, "_e_.getValue()");
		ps.println(prefix + "for (java.util.Map.Entry" + KV + " _e_ : _o_.get" + StringUtils.upper1(varname)
				+ "().entrySet())");
		ps.println(prefix + "	" + thisVar + ".put(" + keycopy + ", " + valuecopy + ");");
	}

	@Override
	public void visit(TypeList type) {
		copyCollection(type, type.getValueType());
	}

	@Override
	public void visit(TypeSet type) {
		copyCollection(type, type.getValueType());
	}

	@Override
	public void visit(TypeVector type) {
		copyCollection(type, type.getValueType());
	}

	@Override
	public void visit(TypeMap type) {
		copyBaseMap(type, type.getKeyType(), type.getValueType());
	}

	@Override
	public void visit(Cbean type) {
		ps.println(prefix + thisVar + " = new " + TypeName.getName(type) + "(_o_.get" + StringUtils.upper1(varname)
				+ "());");
	}

	@Override
	public void visit(Xbean type) {
		ps.println(prefix + thisVar + " = new " + TypeName.getName(type) + "(_o_.get" + StringUtils.upper1(varname)
				+ "());");
	}

	@Override
	public void visit(TypeBoolean type) {
		model1();
	}

	@Override
	public void visit(TypeAny type) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void visit(TypeBinary type) {
		ps.println(prefix + thisVar + " = _o_.get" + StringUtils.upper1(varname) + "OctetsCopy();");
	}

}
